@@ -43,6 +43,10 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
+#include <X11/Xauth.h>
+
+#include "ck-connector.h"
+
 #include "gdm-session-worker.h"
 #include "gdm-marshal.h"
 
@@ -99,6 +103,7 @@ struct GdmSessionWorkerPrivate
 
         int               exit_code;
 
+        CkConnector      *ckc;
         pam_handle_t     *pam_handle;
 
         GPid              child_pid;
@@ -158,6 +163,83 @@ gdm_session_worker_error_quark (void)
         return error_quark;
 }
 
+static gboolean
+open_ck_session (GdmSessionWorker  *worker)
+{
+        struct passwd *pwent;
+        gboolean       ret;
+        int            res;
+        DBusError      error;
+        const char     *display_name;
+        const char     *display_device;
+        const char     *display_hostname;
+        gboolean        is_local;
+
+        if (worker->priv->x11_display_name != NULL) {
+                display_name = worker->priv->x11_display_name;
+        } else {
+                display_name = "";
+        }
+        if (worker->priv->hostname != NULL) {
+                display_hostname = worker->priv->hostname;
+        } else {
+                display_hostname = "";
+        }
+        if (worker->priv->display_device != NULL) {
+                display_device = worker->priv->display_device;
+        } else {
+                display_device = "";
+        }
+
+        g_assert (worker->priv->username != NULL);
+
+        /* FIXME: this isn't very good */
+        if (display_hostname == NULL
+            || display_hostname[0] == '\0'
+            || strcmp (display_hostname, "localhost") == 0) {
+                is_local = TRUE;
+        } else {
+                is_local = FALSE;
+        }
+
+        pwent = getpwnam (worker->priv->username);
+        if (pwent == NULL) {
+                return FALSE;
+        }
+
+        worker->priv->ckc = ck_connector_new ();
+        if (worker->priv->ckc == NULL) {
+                g_warning ("Couldn't create new ConsoleKit connector");
+                goto out;
+        }
+
+        dbus_error_init (&error);
+        res = ck_connector_open_session_with_parameters (worker->priv->ckc,
+                                                         &error,
+                                                         "unix-user", &pwent->pw_uid,
+                                                         "x11-display", &display_name,
+                                                         "x11-display-device", &display_device,
+                                                         "remote-host-name", &display_hostname,
+                                                         "is-local", &is_local,
+                                                         NULL);
+
+        if (! res) {
+                if (dbus_error_is_set (&error)) {
+                        g_warning ("%s\n", error.message);
+                        dbus_error_free (&error);
+                } else {
+                        g_warning ("cannot open CK session: OOM, D-Bus system bus not available,\n"
+                                   "ConsoleKit not available or insufficient privileges.\n");
+                }
+                goto out;
+        }
+
+        ret = TRUE;
+
+ out:
+        return ret;
+}
+
 /* adapted from glib script_execute */
 static void
 script_execute (const gchar *file,
@@ -501,8 +583,8 @@ gdm_session_worker_get_username (GdmSessionWorker  *worker,
 }
 
 static void
-attempt_to_load_user_settings_as_root (GdmSessionWorker *worker,
-                                       const char       *username)
+attempt_to_load_user_settings (GdmSessionWorker *worker,
+                               const char       *username)
 {
         struct passwd *passwd_entry;
         uid_t          old_uid;
@@ -511,9 +593,6 @@ attempt_to_load_user_settings_as_root (GdmSessionWorker *worker,
         old_uid = geteuid ();
         old_gid = getegid ();
 
-        g_assert (old_uid == 0);
-        g_assert (old_gid == 0);
-
         passwd_entry = getpwnam (username);
 
         /* User input isn't a valid username
@@ -522,19 +601,18 @@ attempt_to_load_user_settings_as_root (GdmSessionWorker *worker,
                 return;
         }
 
+        /* We may get called late in the pam conversation after
+         * the user has already been authenticated.  This could
+         * happen if for instance, the user's home directory isn't
+         * available until late in the pam conversation so user
+         * settings couldn't get loaded until late in the conversation.
+         * If we get called late the seteuid/setgid calls here will fail,
+         * but that's okay, because we'll already be the uid/gid we want
+         * to be.
+         */
         setegid (passwd_entry->pw_gid);
         seteuid (passwd_entry->pw_uid);
 
-        /* FIXME: if this call fails, we should probably do some
-         * heuristics to try to figure out if .dmrc doesn't
-         * exist at all or just isn't readable yet.  If it doesn't
-         * exist at all (and won't exist by the end of the pam
-         * conversation) then we should send some sort of
-         * "NoSavedSettings" message to the slave so that it can
-         * tell the greeter to use specific default settings
-         * instead of the place holder "Last Session" "Last
-         * Language" items in the combo boxes in the panel
-         */
         gdm_session_settings_load (worker->priv->user_settings,
                                    passwd_entry->pw_dir,
                                    NULL);
@@ -566,7 +644,7 @@ gdm_session_worker_update_username (GdmSessionWorker *worker)
                  */
                 if (username != NULL &&
                     !gdm_session_settings_is_loaded (worker->priv->user_settings)) {
-                        attempt_to_load_user_settings_as_root (worker, username);
+                        attempt_to_load_user_settings (worker, username);
                 }
 
                 if ((worker->priv->username == username) ||
@@ -929,6 +1007,37 @@ _get_tty_for_pam (const char *x11_display_name,
 #endif
 }
 
+#ifdef PAM_XAUTHDATA
+static struct pam_xauth_data *
+_get_xauth_for_pam (const char *x11_authority_file)
+{
+        FILE                  *fh;
+        Xauth                 *auth = NULL;
+        struct pam_xauth_data *retval = NULL;
+        gsize                  len = sizeof (*retval) + 1;
+
+        fh = fopen (x11_authority_file, "r");
+        if (fh) {
+                auth = XauReadAuth (fh);
+                fclose (fh);
+        }
+        if (auth) {
+                len += auth->name_length + auth->data_length;
+                retval = g_malloc0 (len);
+        }
+        if (retval) {
+                retval->namelen = auth->name_length;
+                retval->name = (char *) (retval + 1);
+                memcpy (retval->name, auth->name, auth->name_length);
+                retval->datalen = auth->data_length;
+                retval->data = retval->name + auth->name_length + 1;
+                memcpy (retval->data, auth->data, auth->data_length);
+        }
+        XauDisposeAuth (auth);
+        return retval;
+}
+#endif
+
 static gboolean
 gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                                    const char       *service,
@@ -939,9 +1048,12 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                                    const char       *display_device,
                                    GError          **error)
 {
-        struct pam_conv  pam_conversation;
-        int              error_code;
-        char            *pam_tty;
+        struct pam_conv        pam_conversation;
+#ifdef PAM_XAUTHDATA
+        struct pam_xauth_data *pam_xauth;
+#endif
+        int                    error_code;
+        char                  *pam_tty;
 
         g_assert (worker->priv->pam_handle == NULL);
 
@@ -1015,6 +1127,35 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                 goto out;
         }
 
+#ifdef PAM_XDISPLAY
+        /* set XDISPLAY */
+        error_code = pam_set_item (worker->priv->pam_handle, PAM_XDISPLAY, x11_display_name);
+
+        if (error_code != PAM_SUCCESS) {
+                g_set_error (error,
+                             GDM_SESSION_WORKER_ERROR,
+                             GDM_SESSION_WORKER_ERROR_AUTHENTICATING,
+                             _("error informing authentication system of display string - %s"),
+                             pam_strerror (worker->priv->pam_handle, error_code));
+                goto out;
+        }
+#endif
+#ifdef PAM_XAUTHDATA
+        /* set XAUTHDATA */
+        pam_xauth = _get_xauth_for_pam (x11_authority_file);
+        error_code = pam_set_item (worker->priv->pam_handle, PAM_XAUTHDATA, pam_xauth);
+        g_free (pam_xauth);
+
+        if (error_code != PAM_SUCCESS) {
+                g_set_error (error,
+                             GDM_SESSION_WORKER_ERROR,
+                             GDM_SESSION_WORKER_ERROR_AUTHENTICATING,
+                             _("error informing authentication system of display xauth credentials - %s"),
+                             pam_strerror (worker->priv->pam_handle, error_code));
+                goto out;
+        }
+#endif
+
         g_debug ("GdmSessionWorker: state SETUP_COMPLETE");
         worker->priv->state = GDM_SESSION_WORKER_STATE_SETUP_COMPLETE;
 
@@ -1418,6 +1559,24 @@ gdm_session_worker_get_environment (GdmSessionWorker *worker)
         return (char **) g_ptr_array_free (environment, FALSE);
 }
 
+static void
+register_ck_session (GdmSessionWorker *worker)
+{
+        const char *session_cookie;
+        gboolean    res;
+
+        session_cookie = NULL;
+        res = open_ck_session (worker);
+        if (res) {
+                session_cookie = ck_connector_get_cookie (worker->priv->ckc);
+        }
+        if (session_cookie != NULL) {
+                gdm_session_worker_set_environment_variable (worker,
+                                                             "XDG_SESSION_COOKIE",
+                                                             session_cookie);
+        }
+}
+
 static void
 session_worker_child_watch (GPid              pid,
                             int               status,
@@ -1446,6 +1605,12 @@ session_worker_child_watch (GPid              pid,
                                       num);
         }
 
+        if (worker->priv->ckc != NULL) {
+                ck_connector_close_session (worker->priv->ckc, NULL);
+                ck_connector_unref (worker->priv->ckc);
+                worker->priv->ckc = NULL;
+        }
+
         gdm_session_worker_uninitialize_pam (worker, PAM_SUCCESS);
 
         worker->priv->child_pid = -1;
@@ -1480,6 +1645,15 @@ _open_session_log (const char *dir)
         char *filename;
 
         filename = g_build_filename (dir, GDM_SESSION_LOG_FILENAME, NULL);
+
+        if (g_access (dir, R_OK | W_OK | X_OK) == 0 && g_access (filename, R_OK | W_OK) == 0) {
+                char *filename_old;
+
+                filename_old = g_strdup_printf ("%s.old", filename);
+                g_rename (filename, filename_old);
+                g_free (filename_old);
+        }
+
         fd = g_open (filename, O_RDWR | O_APPEND | O_CREAT, 0600);
 
         if (fd < 0 || !_fd_is_normal_file (fd)) {
@@ -1555,6 +1729,8 @@ gdm_session_worker_start_user_session (GdmSessionWorker  *worker,
         g_debug ("GdmSessionWorker: querying pam for user environment");
         gdm_session_worker_update_environment_from_pam (worker);
 
+        register_ck_session (worker);
+
         g_debug ("GdmSessionWorker: opening user session with program '%s'",
                  worker->priv->arguments[0]);
 
@@ -1784,6 +1960,14 @@ gdm_session_worker_set_language_name (GdmSessionWorker *worker,
                                                 language_name);
 }
 
+static void
+gdm_session_worker_set_layout_name (GdmSessionWorker *worker,
+                                    const char       *layout_name)
+{
+        gdm_session_settings_set_layout_name (worker->priv->user_settings,
+                                              layout_name);
+}
+
 static void
 on_set_language_name (GdmSessionWorker *worker,
                       DBusMessage      *message)
@@ -1806,6 +1990,28 @@ on_set_language_name (GdmSessionWorker *worker,
         }
 }
 
+static void
+on_set_layout_name (GdmSessionWorker *worker,
+                    DBusMessage      *message)
+{
+        DBusError   error;
+        const char *layout_name;
+        dbus_bool_t res;
+
+        dbus_error_init (&error);
+        res = dbus_message_get_args (message,
+                                     &error,
+                                     DBUS_TYPE_STRING, &layout_name,
+                                     DBUS_TYPE_INVALID);
+        if (res) {
+                g_debug ("GdmSessionWorker: layout name set to %s", layout_name);
+                gdm_session_worker_set_layout_name (worker, layout_name);
+        } else {
+                g_warning ("Unable to get arguments: %s", error.message);
+                dbus_error_free (&error);
+        }
+}
+
 static void
 on_saved_language_name_read (GdmSessionWorker *worker)
 {
@@ -1818,6 +2024,18 @@ on_saved_language_name_read (GdmSessionWorker *worker)
         g_free (language_name);
 }
 
+static void
+on_saved_layout_name_read (GdmSessionWorker *worker)
+{
+        char *layout_name;
+
+        layout_name = gdm_session_settings_get_layout_name (worker->priv->user_settings);
+        send_dbus_string_method (worker->priv->connection,
+                                 "SavedLayoutNameRead",
+                                 layout_name);
+        g_free (layout_name);
+}
+
 static void
 on_saved_session_name_read (GdmSessionWorker *worker)
 {
@@ -1843,6 +2061,11 @@ do_setup (GdmSessionWorker *worker)
                                   G_CALLBACK (on_saved_language_name_read),
                                   worker);
 
+        g_signal_connect_swapped (worker->priv->user_settings,
+                                  "notify::layout-name",
+                                  G_CALLBACK (on_saved_layout_name_read),
+                                  worker);
+
         g_signal_connect_swapped (worker->priv->user_settings,
                                   "notify::session-name",
                                   G_CALLBACK (on_saved_session_name_read),
@@ -1859,8 +2082,8 @@ do_setup (GdmSessionWorker *worker)
          * We'll try now, and if it doesn't work out, try later.
          */
         if (worker->priv->username != NULL) {
-                attempt_to_load_user_settings_as_root (worker,
-                                                       worker->priv->username);
+                attempt_to_load_user_settings (worker,
+                                               worker->priv->username);
         }
 
         error = NULL;
@@ -2300,6 +2523,8 @@ worker_dbus_handle_message (DBusConnection *connection,
                 on_set_environment_variable (worker, message);
         } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetLanguageName")) {
                 on_set_language_name (worker, message);
+        } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetLayoutName")) {
+                on_set_layout_name (worker, message);
         } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetSessionName")) {
                 on_set_session_name (worker, message);
         } else {
@@ -2466,3 +2691,4 @@ gdm_session_worker_new (const char *address)
 
         return GDM_SESSION_WORKER (object);
 }
+
