@@ -48,6 +48,8 @@
 
 #define DBUS_LAUNCH_COMMAND BINDIR "/dbus-launch --exit-with-session"
 
+#define MAX_LOGS 5
+
 extern char **environ;
 
 #define GDM_WELCOME_SESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_WELCOME_SESSION, GdmWelcomeSessionPrivate))
@@ -376,6 +378,33 @@ welcome_session_child_watch (GPid               pid,
         stop_dbus_daemon (session);
 }
 
+static void
+rotate_logs (const char *path,
+             guint       n_copies)
+{
+        int i;
+
+        for (i = n_copies - 1; i > 0; i--) {
+                char *name_n;
+                char *name_n1;
+
+                name_n = g_strdup_printf ("%s.%d", path, i);
+                if (i > 1) {
+                        name_n1 = g_strdup_printf ("%s.%d", path, i - 1);
+                } else {
+                        name_n1 = g_strdup (path);
+                }
+
+                VE_IGNORE_EINTR (g_unlink (name_n));
+                VE_IGNORE_EINTR (g_rename (name_n1, name_n));
+
+                g_free (name_n1);
+                g_free (name_n);
+        }
+
+        VE_IGNORE_EINTR (g_unlink (path));
+}
+
 typedef struct {
         const char *user_name;
         const char *group_name;
@@ -450,6 +479,8 @@ spawn_child_setup (SpawnChildData *data)
         if (data->log_file != NULL) {
                 int logfd;
 
+                rotate_logs (data->log_file, MAX_LOGS);
+
                 VE_IGNORE_EINTR (g_unlink (data->log_file));
                 VE_IGNORE_EINTR (logfd = open (data->log_file, O_CREAT|O_TRUNC|O_WRONLY|O_EXCL, 0644));
 
@@ -610,7 +641,7 @@ parse_dbus_launch_output (const char *output,
         error = NULL;
         re = g_regex_new ("DBUS_SESSION_BUS_ADDRESS=(.+)\nDBUS_SESSION_BUS_PID=([0-9]+)", 0, 0, &error);
         if (re == NULL) {
-                g_critical (error->message);
+                g_critical ("%s", error->message);
         }
 
         g_regex_match (re, output, 0, &match_info);
@@ -705,15 +736,15 @@ gdm_welcome_session_spawn (GdmWelcomeSession *welcome_session)
 
         g_debug ("GdmWelcomeSession: Running welcome_session process: %s", welcome_session->priv->command);
 
+        if (welcome_session->priv->register_ck_session) {
+                open_welcome_session (welcome_session);
+        }
+
         res = start_dbus_daemon (welcome_session);
         if (! res) {
                 /* FIXME: */
         }
 
-        if (welcome_session->priv->register_ck_session) {
-                open_welcome_session (welcome_session);
-        }
-
         env = get_welcome_environment (welcome_session);
 
         error = NULL;
@@ -1113,14 +1144,14 @@ gdm_welcome_session_class_init (GdmWelcomeSessionClass *klass)
                                          g_param_spec_string ("user-name",
                                                               "user name",
                                                               "user name",
-                                                              "gdm",
+                                                              GDM_USERNAME,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
         g_object_class_install_property (object_class,
                                          PROP_GROUP_NAME,
                                          g_param_spec_string ("group-name",
                                                               "group name",
                                                               "group name",
-                                                              "gdm",
+                                                              GDM_GROUPNAME,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
         g_object_class_install_property (object_class,
                                          PROP_SERVER_ADDRESS,
