@@ -40,21 +40,16 @@
 #include "gdm-language-option-widget.h"
 #include "gdm-layout-option-widget.h"
 #include "gdm-session-option-widget.h"
-#include "gdm-a11y-preferences-dialog.h"
 #include "gdm-profile.h"
 
 #include "na-tray.h"
 
-#define KEY_DISABLE_A11Y_BUTTON "/apps/gdm/simple-greeter/disable_accessibility_button"
-
 #define GDM_GREETER_PANEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_PANEL, GdmGreeterPanelPrivate))
 
 struct GdmGreeterPanelPrivate
 {
         int                     monitor;
         GdkRectangle            geometry;
-        GtkWidget              *a11y_button;
-        GtkWidget              *a11y_dialog;
         GtkWidget              *hbox;
         GtkWidget              *alignment;
         GtkWidget              *option_hbox;
@@ -70,6 +65,7 @@ struct GdmGreeterPanelPrivate
 
 enum {
         PROP_0,
+        PROP_MONITOR
 };
 
 enum {
@@ -87,13 +83,37 @@ static void     gdm_greeter_panel_finalize    (GObject              *object);
 
 G_DEFINE_TYPE (GdmGreeterPanel, gdm_greeter_panel, GTK_TYPE_WINDOW)
 
+static void
+gdm_greeter_panel_set_monitor (GdmGreeterPanel *panel,
+                               int              monitor)
+{
+        g_return_if_fail (GDM_IS_GREETER_PANEL (panel));
+
+        if (panel->priv->monitor == monitor) {
+                return;
+        }
+
+        panel->priv->monitor = monitor;
+
+        gtk_widget_queue_resize (GTK_WIDGET (panel));
+
+        g_object_notify (G_OBJECT (panel), "monitor");
+}
+
 static void
 gdm_greeter_panel_set_property (GObject        *object,
                                 guint           prop_id,
                                 const GValue   *value,
                                 GParamSpec     *pspec)
 {
+        GdmGreeterPanel *self;
+
+        self = GDM_GREETER_PANEL (object);
+
         switch (prop_id) {
+        case PROP_MONITOR:
+                gdm_greeter_panel_set_monitor (self, g_value_get_int (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -106,7 +126,14 @@ gdm_greeter_panel_get_property (GObject        *object,
                                 GValue         *value,
                                 GParamSpec     *pspec)
 {
+        GdmGreeterPanel *self;
+
+        self = GDM_GREETER_PANEL (object);
+
         switch (prop_id) {
+        case PROP_MONITOR:
+                g_value_set_int (value, self->priv->monitor);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -424,6 +451,16 @@ gdm_greeter_panel_class_init (GdmGreeterPanelClass *klass)
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
 
+        g_object_class_install_property (object_class,
+                                         PROP_MONITOR,
+                                         g_param_spec_int ("monitor",
+                                                           "Xinerama monitor",
+                                                           "The monitor (in terms of Xinerama) which the window is on",
+                                                           0,
+                                                           G_MAXINT,
+                                                           0,
+                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
         g_type_class_add_private (klass, sizeof (GdmGreeterPanelPrivate));
 }
 
@@ -458,8 +495,8 @@ on_layout_activated (GdmLayoutOptionWidget *widget,
                 return;
         }
 
-	g_debug ("GdmGreeterPanel: activating selected layout %s", layout);
-	gdm_layout_activate (layout);
+        g_debug ("GdmGreeterPanel: activating selected layout %s", layout);
+        gdm_layout_activate (layout);
 
         g_signal_emit (panel, signals[LAYOUT_SELECTED], 0, layout);
 
@@ -483,59 +520,11 @@ on_session_activated (GdmSessionOptionWidget *widget,
         g_free (session);
 }
 
-static void
-on_a11y_dialog_response (GtkDialog       *dialog,
-                         int              response,
-                         GdmGreeterPanel *panel)
-{
-        g_signal_handlers_disconnect_by_func (dialog,
-                                              on_a11y_dialog_response,
-                                              panel);
-
-        gtk_widget_destroy (GTK_WIDGET (dialog));
-        panel->priv->a11y_dialog = NULL;
-}
-
-static void
-on_a11y_button_clicked (GtkButton       *button,
-                        GdmGreeterPanel *panel)
-{
-        if (panel->priv->a11y_dialog == NULL) {
-                panel->priv->a11y_dialog = gdm_a11y_preferences_dialog_new ();
-                g_signal_connect (panel->priv->a11y_dialog,
-                                  "response",
-                                  G_CALLBACK (on_a11y_dialog_response),
-                                  panel);
-        }
-        gtk_window_present (GTK_WINDOW (panel->priv->a11y_dialog));
-}
-
-static gboolean
-is_a11y_button_disabled (GdmGreeterPanel *panel)
-{
-        GConfClient *client;
-        GError      *error;
-        gboolean     disabled;
-
-        client = gconf_client_get_default ();
-        error = NULL;
-        disabled = gconf_client_get_bool (client, KEY_DISABLE_A11Y_BUTTON, &error);
-        if (error != NULL) {
-                g_debug ("GdmGreeterPanel: unable to get disable-accessibility-button configuration: %s", error->message);
-                g_error_free (error);
-        }
-        g_object_unref (client);
-
-        return disabled;
-}
-
 static void
 gdm_greeter_panel_init (GdmGreeterPanel *panel)
 {
         NaTray    *tray;
-        GtkWidget *image;
         GtkWidget *spacer;
-	AtkObject *atk_obj;
 
         gdm_profile_start (NULL);
 
@@ -560,23 +549,6 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
         gtk_widget_show (panel->priv->hbox);
         gtk_container_add (GTK_CONTAINER (panel), panel->priv->hbox);
 
-        if (! is_a11y_button_disabled (panel)) {
-                panel->priv->a11y_button = gtk_button_new ();
-                image = gtk_image_new_from_icon_name ("preferences-desktop-accessibility", GTK_ICON_SIZE_BUTTON);
-                gtk_container_add (GTK_CONTAINER (panel->priv->a11y_button), image);
-		atk_obj = gtk_widget_get_accessible (panel->priv->a11y_button);
-		if (GTK_IS_ACCESSIBLE (atk_obj))
-			atk_object_set_name (atk_obj, _("Accessibility Preferences"));
-
-                gtk_widget_show (image);
-                gtk_widget_show (panel->priv->a11y_button);
-                g_signal_connect (G_OBJECT (panel->priv->a11y_button),
-                                  "clicked",
-                                  G_CALLBACK (on_a11y_button_clicked), panel);
-
-                gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->a11y_button, FALSE, FALSE, 0);
-        }
-
         panel->priv->alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
         gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->alignment, TRUE, TRUE, 0);
         gtk_widget_show (panel->priv->alignment);
@@ -652,11 +624,14 @@ gdm_greeter_panel_finalize (GObject *object)
 }
 
 GtkWidget *
-gdm_greeter_panel_new (void)
+gdm_greeter_panel_new (GdkScreen *screen,
+                       int        monitor)
 {
         GObject *object;
 
         object = g_object_new (GDM_TYPE_GREETER_PANEL,
+                               "screen", screen,
+                               "monitor", monitor,
                                NULL);
 
         return GTK_WIDGET (object);
@@ -677,8 +652,8 @@ gdm_greeter_panel_hide_user_options (GdmGreeterPanel *panel)
         gtk_widget_hide (panel->priv->language_option_widget);
         gtk_widget_hide (panel->priv->layout_option_widget);
 
-	g_debug ("GdmGreeterPanel: activating default layout");
-	gdm_layout_activate (NULL); 
+        g_debug ("GdmGreeterPanel: activating default layout");
+        gdm_layout_activate (NULL);
 }
 
 void
@@ -734,8 +709,8 @@ gdm_greeter_panel_set_default_layout_name (GdmGreeterPanel *panel,
         gdm_option_widget_set_default_item (GDM_OPTION_WIDGET (panel->priv->layout_option_widget),
                                             layout_name);
 
-	g_debug ("GdmGreeterPanel: activating layout: %s", layout_name);
-	gdm_layout_activate (layout_name);
+        g_debug ("GdmGreeterPanel: activating layout: %s", layout_name);
+        gdm_layout_activate (layout_name);
 #endif
 }
 
