@@ -50,25 +50,33 @@
 struct GdmOptionWidgetPrivate
 {
         GtkWidget                *label;
+        GtkWidget                *image;
         char                     *label_text;
+        char                     *icon_name;
+        char                     *default_item_id;
 
         GtkWidget                *items_combo_box;
         GtkListStore             *list_store;
 
+        GtkTreeModelFilter       *model_filter;
         GtkTreeModelSort         *model_sorter;
 
         GtkTreeRowReference      *active_row;
-        GtkTreeRowReference      *separator_row;
+        GtkTreeRowReference      *top_separator_row;
+        GtkTreeRowReference      *bottom_separator_row;
 
-        gint                     number_of_normal_rows;
-        gint                     number_of_separated_rows;
+        gint                     number_of_top_rows;
+        gint                     number_of_middle_rows;
+        gint                     number_of_bottom_rows;
 
-        GdmOptionWidgetPosition  separator_position;
+        guint                    check_idle_id;
 };
 
 enum {
         PROP_0,
-        PROP_LABEL_TEXT
+        PROP_LABEL_TEXT,
+        PROP_ICON_NAME,
+        PROP_DEFAULT_ITEM
 };
 
 enum {
@@ -86,7 +94,7 @@ G_DEFINE_TYPE (GdmOptionWidget, gdm_option_widget, GTK_TYPE_ALIGNMENT)
 enum {
         OPTION_NAME_COLUMN = 0,
         OPTION_COMMENT_COLUMN,
-        OPTION_ITEM_IS_SEPARATED_COLUMN,
+        OPTION_POSITION_COLUMN,
         OPTION_ID_COLUMN,
         NUMBER_OF_OPTION_COLUMNS
 };
@@ -166,12 +174,17 @@ static void
 activate_from_item_id (GdmOptionWidget *widget,
                        const char      *item_id)
 {
-        GtkTreeModel *model;
-        GtkTreePath  *path;
         GtkTreeIter   iter;
 
-        model = GTK_TREE_MODEL (widget->priv->list_store);
-        path = NULL;
+        if (item_id == NULL) {
+                if (widget->priv->active_row != NULL) {
+                    gtk_tree_row_reference_free (widget->priv->active_row);
+                    widget->priv->active_row = NULL;
+                }
+
+                gtk_combo_box_set_active (GTK_COMBO_BOX (widget->priv->items_combo_box), -1);
+                return;
+        }
 
         if (!find_item (widget, item_id, &iter)) {
                 g_critical ("Tried to activate non-existing item from option widget");
@@ -251,11 +264,43 @@ gdm_option_widget_set_active_item (GdmOptionWidget *widget,
                                    const char      *id)
 {
         g_return_if_fail (GDM_IS_OPTION_WIDGET (widget));
-        g_return_if_fail (id != NULL);
 
         activate_from_item_id (widget, id);
 }
 
+char *
+gdm_option_widget_get_default_item (GdmOptionWidget *widget)
+{
+        g_return_val_if_fail (GDM_IS_OPTION_WIDGET (widget), NULL);
+
+        return g_strdup (widget->priv->default_item_id);
+}
+
+void
+gdm_option_widget_set_default_item (GdmOptionWidget *widget,
+                                    const char      *item)
+{
+        g_return_if_fail (GDM_IS_OPTION_WIDGET (widget));
+        g_return_if_fail (item == NULL ||
+                          gdm_option_widget_lookup_item (widget, item,
+                                                         NULL, NULL, NULL));
+
+        if (widget->priv->default_item_id == NULL ||
+            item == NULL ||
+            strcmp (widget->priv->default_item_id, item) != 0) {
+                g_free (widget->priv->default_item_id);
+                widget->priv->default_item_id = NULL;
+
+                if (widget->priv->active_row == NULL || item != NULL) {
+                    activate_from_item_id (widget, item);
+                }
+
+                widget->priv->default_item_id = g_strdup (item);
+
+                g_object_notify (G_OBJECT (widget), "default-item");
+        }
+}
+
 static const char *
 gdm_option_widget_get_label_text (GdmOptionWidget *widget)
 {
@@ -276,11 +321,45 @@ gdm_option_widget_set_label_text (GdmOptionWidget *widget,
         }
 }
 
+static const char *
+gdm_option_widget_get_icon_name (GdmOptionWidget *widget)
+{
+        return widget->priv->icon_name;
+}
+
+static void
+gdm_option_widget_set_icon_name (GdmOptionWidget *widget,
+                                 const char      *name)
+{
+        if (name == NULL && widget->priv->icon_name != NULL) {
+                /* remove icon */
+                g_free (widget->priv->icon_name);
+                widget->priv->icon_name = NULL;
+                gtk_widget_hide (widget->priv->image);
+                gtk_image_clear (GTK_IMAGE (widget->priv->image));
+                g_object_notify (G_OBJECT (widget), "icon-name");
+        } else if (name != NULL && widget->priv->icon_name == NULL) {
+                /* add icon */
+                widget->priv->icon_name = g_strdup (name);
+                gtk_widget_show (widget->priv->image);
+                gtk_image_set_from_icon_name (GTK_IMAGE (widget->priv->image), name, GTK_ICON_SIZE_BUTTON);
+                g_object_notify (G_OBJECT (widget), "icon-name");
+        } else if (name != NULL
+                   && widget->priv->icon_name != NULL
+                   && strcmp (widget->priv->icon_name, name) != 0) {
+                /* changed icon */
+                g_free (widget->priv->icon_name);
+                widget->priv->icon_name = g_strdup (name);
+                gtk_image_set_from_icon_name (GTK_IMAGE (widget->priv->image), name, GTK_ICON_SIZE_BUTTON);
+                g_object_notify (G_OBJECT (widget), "icon-name");
+        }
+}
+
 static void
 gdm_option_widget_set_property (GObject        *object,
-                                 guint           prop_id,
-                                 const GValue   *value,
-                                 GParamSpec     *pspec)
+                                guint           prop_id,
+                                const GValue   *value,
+                                GParamSpec     *pspec)
 {
         GdmOptionWidget *self;
 
@@ -290,7 +369,12 @@ gdm_option_widget_set_property (GObject        *object,
         case PROP_LABEL_TEXT:
                 gdm_option_widget_set_label_text (self, g_value_get_string (value));
                 break;
-
+        case PROP_ICON_NAME:
+                gdm_option_widget_set_icon_name (self, g_value_get_string (value));
+                break;
+        case PROP_DEFAULT_ITEM:
+                gdm_option_widget_set_default_item (self, g_value_get_string (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -312,6 +396,14 @@ gdm_option_widget_get_property (GObject        *object,
                 g_value_set_string (value,
                                     gdm_option_widget_get_label_text (self));
                 break;
+        case PROP_ICON_NAME:
+                g_value_set_string (value,
+                                    gdm_option_widget_get_icon_name (self));
+                break;
+        case PROP_DEFAULT_ITEM:
+                g_value_take_string (value,
+                                    gdm_option_widget_get_default_item (self));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -320,13 +412,10 @@ gdm_option_widget_get_property (GObject        *object,
 
 static GObject *
 gdm_option_widget_constructor (GType                  type,
-                                guint                  n_construct_properties,
-                                GObjectConstructParam *construct_properties)
+                               guint                  n_construct_properties,
+                               GObjectConstructParam *construct_properties)
 {
         GdmOptionWidget      *option_widget;
-        GdmOptionWidgetClass *klass;
-
-        klass = GDM_OPTION_WIDGET_CLASS (g_type_class_peek (GDM_TYPE_OPTION_WIDGET));
 
         option_widget = GDM_OPTION_WIDGET (G_OBJECT_CLASS (gdm_option_widget_parent_class)->constructor (type,
                                                                                                          n_construct_properties,
@@ -342,9 +431,14 @@ gdm_option_widget_dispose (GObject *object)
 
         widget = GDM_OPTION_WIDGET (object);
 
-        if (widget->priv->separator_row != NULL) {
-                gtk_tree_row_reference_free (widget->priv->separator_row);
-                widget->priv->separator_row = NULL;
+        if (widget->priv->top_separator_row != NULL) {
+                gtk_tree_row_reference_free (widget->priv->top_separator_row);
+                widget->priv->top_separator_row = NULL;
+        }
+
+        if (widget->priv->bottom_separator_row != NULL) {
+                gtk_tree_row_reference_free (widget->priv->bottom_separator_row);
+                widget->priv->bottom_separator_row = NULL;
         }
 
         if (widget->priv->active_row != NULL) {
@@ -355,16 +449,31 @@ gdm_option_widget_dispose (GObject *object)
         G_OBJECT_CLASS (gdm_option_widget_parent_class)->dispose (object);
 }
 
+static gboolean
+gdm_option_widget_mnemonic_activate (GtkWidget *widget,
+                                     gboolean   group_cycling)
+{
+        GdmOptionWidget *option_widget;
+
+        option_widget = GDM_OPTION_WIDGET (widget);
+        gtk_widget_grab_focus (option_widget->priv->items_combo_box);
+        gtk_combo_box_popup (GTK_COMBO_BOX (option_widget->priv->items_combo_box));
+
+        return TRUE;
+}
+
 static void
 gdm_option_widget_class_init (GdmOptionWidgetClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
         object_class->get_property = gdm_option_widget_get_property;
         object_class->set_property = gdm_option_widget_set_property;
         object_class->constructor = gdm_option_widget_constructor;
         object_class->dispose = gdm_option_widget_dispose;
         object_class->finalize = gdm_option_widget_finalize;
+        widget_class->mnemonic_activate = gdm_option_widget_mnemonic_activate;
 
         gtk_rc_parse_string (GDM_OPTION_WIDGET_RC_STRING);
 
@@ -386,7 +495,22 @@ gdm_option_widget_class_init (GdmOptionWidgetClass *klass)
                                                               NULL,
                                                               (G_PARAM_READWRITE |
                                                                G_PARAM_CONSTRUCT)));
+        g_object_class_install_property (object_class,
+                                         PROP_ICON_NAME,
+                                         g_param_spec_string ("icon-name",
+                                                              _("Icon name"),
+                                                              _("The icon to use with the label"),
+                                                              NULL,
+                                                              (G_PARAM_READWRITE |
+                                                               G_PARAM_CONSTRUCT)));
 
+        g_object_class_install_property (object_class,
+                                         PROP_DEFAULT_ITEM,
+                                         g_param_spec_string ("default-item",
+                                                              _("Default Item"),
+                                                              _("The id of the default item"),
+                                                              NULL,
+                                                              G_PARAM_READWRITE));
 
         g_type_class_add_private (klass, sizeof (GdmOptionWidgetPrivate));
 }
@@ -395,127 +519,236 @@ static void
 on_changed (GtkComboBox     *combo_box,
             GdmOptionWidget *widget)
 {
+        if (widget->priv->default_item_id == NULL) {
+                return;
+        }
+
         activate_selected_item (widget);
 }
 
+static void
+on_default_item_changed (GdmOptionWidget *widget)
+{
+        gtk_widget_set_sensitive (widget->priv->items_combo_box,
+                                  widget->priv->default_item_id != NULL);
+        gtk_tree_model_filter_refilter (widget->priv->model_filter);
+}
+
+static gboolean
+path_is_row (GdmOptionWidget     *widget,
+             GtkTreeModel        *model,
+             GtkTreePath         *path,
+             GtkTreeRowReference *row)
+{
+        GtkTreePath      *row_path;
+        GtkTreePath      *translated_path;
+        gboolean          is_row;
+
+        row_path = gtk_tree_row_reference_get_path (row);
+
+        if (row_path == NULL) {
+                return FALSE;
+        }
+
+        if (model == GTK_TREE_MODEL (widget->priv->model_sorter)) {
+                GtkTreePath *filtered_path;
+
+                filtered_path = gtk_tree_model_sort_convert_path_to_child_path (widget->priv->model_sorter, path);
+
+                translated_path = gtk_tree_model_filter_convert_path_to_child_path (widget->priv->model_filter, filtered_path);
+                gtk_tree_path_free (filtered_path);
+        } else if (model == GTK_TREE_MODEL (widget->priv->model_filter)) {
+                translated_path = gtk_tree_model_filter_convert_path_to_child_path (widget->priv->model_filter, path);
+        } else {
+                g_assert (model == GTK_TREE_MODEL (widget->priv->list_store));
+                translated_path = gtk_tree_path_copy (path);
+        }
+
+        if (gtk_tree_path_compare (row_path, translated_path) == 0) {
+                is_row = TRUE;
+        } else {
+                is_row = FALSE;
+        }
+        gtk_tree_path_free (translated_path);
+
+        return is_row;
+}
+
+static gboolean
+path_is_top_separator (GdmOptionWidget *widget,
+                       GtkTreeModel    *model,
+                       GtkTreePath     *path)
+{
+        if (widget->priv->top_separator_row != NULL) {
+                if (path_is_row (widget, model, path,
+                                 widget->priv->top_separator_row)) {
+                    return TRUE;
+                }
+        }
+
+        return FALSE;
+}
+
+static gboolean
+path_is_bottom_separator (GdmOptionWidget *widget,
+                          GtkTreeModel    *model,
+                          GtkTreePath     *path)
+{
+        if (widget->priv->bottom_separator_row != NULL) {
+
+                if (path_is_row (widget, model, path,
+                                 widget->priv->bottom_separator_row)) {
+                    return TRUE;
+                }
+        }
+
+        return FALSE;
+}
+
 static gboolean
 path_is_separator (GdmOptionWidget *widget,
-                   GtkTreeModel     *model,
-                   GtkTreePath      *path)
+                   GtkTreeModel    *model,
+                   GtkTreePath     *path)
 {
-        GtkTreePath      *base_path;
-        GtkTreePath      *sorted_path;
-        GtkTreePath      *separator_path;
-        gboolean          is_separator;
-
-        if (widget->priv->separator_row == NULL) {
-                return FALSE;
-        }
-
-        base_path = gtk_tree_row_reference_get_path (widget->priv->separator_row);
-        separator_path = base_path;
-        sorted_path = NULL;
-
-        if (base_path == NULL) {
-                return FALSE;
-        }
-
-        if (model != GTK_TREE_MODEL (widget->priv->list_store)) {
-                sorted_path = gtk_tree_model_sort_convert_child_path_to_path (widget->priv->model_sorter, base_path);
-                separator_path = sorted_path;
-
-                gtk_tree_path_free (base_path);
-                base_path = NULL;
-        }
-
-        if ((separator_path != NULL) &&
-            gtk_tree_path_compare (path, separator_path) == 0) {
-                is_separator = TRUE;
-        } else {
-                is_separator = FALSE;
-        }
-        gtk_tree_path_free (separator_path);
-
-        return is_separator;
+        return path_is_top_separator (widget, model, path) ||
+               path_is_bottom_separator (widget, model, path);
 }
 
-static int
-compare_item  (GtkTreeModel *model,
-               GtkTreeIter  *a,
-               GtkTreeIter  *b,
-               gpointer      data)
+static gboolean
+gdm_option_widget_check_visibility (GdmOptionWidget *widget)
+{
+        if (widget->priv->number_of_top_rows > 0 ||
+            widget->priv->number_of_middle_rows > 1 || 
+            widget->priv->number_of_bottom_rows > 0) {
+                gtk_widget_show (widget->priv->items_combo_box);
+                gtk_widget_show (widget->priv->label);
+
+                if (widget->priv->icon_name != NULL) {
+                        gtk_widget_show (widget->priv->image);
+                }
+        } else {
+                gtk_widget_hide (widget->priv->items_combo_box);
+                gtk_widget_hide (widget->priv->label);
+                gtk_widget_hide (widget->priv->image);
+        }
+
+        widget->priv->check_idle_id = 0;
+        return FALSE;
+}
+
+static void
+gdm_option_widget_queue_visibility_check (GdmOptionWidget *widget)
+{
+        if (widget->priv->check_idle_id == 0) {
+                widget->priv->check_idle_id = g_idle_add ((GSourceFunc) gdm_option_widget_check_visibility, widget);
+        }
+}
+
+static gboolean
+check_item_visibilty (GtkTreeModel *model,
+                      GtkTreeIter  *iter,
+                      gpointer      data)
 {
         GdmOptionWidget *widget;
-        char             *name_a;
-        char             *name_b;
-        gboolean          is_separate_a;
-        gboolean          is_separate_b;
-        int               result;
-        int               direction;
-        GtkTreeIter      *separator_iter;
+        GtkTreePath     *path;
+        gboolean         is_top_separator;
+        gboolean         is_bottom_separator;
+        gboolean         is_visible;
 
         g_assert (GDM_IS_OPTION_WIDGET (data));
 
         widget = GDM_OPTION_WIDGET (data);
 
-        separator_iter = NULL;
-        if (widget->priv->separator_row != NULL) {
+        path = gtk_tree_model_get_path (model, iter);
+        is_top_separator = path_is_top_separator (widget, model, path);
+        is_bottom_separator = path_is_bottom_separator (widget, model, path);
+        gtk_tree_path_free (path);
 
-                GtkTreePath      *path_a;
-                GtkTreePath      *path_b;
-
-                path_a = gtk_tree_model_get_path (model, a);
-                path_b = gtk_tree_model_get_path (model, b);
-
-                if (path_is_separator (widget, model, path_a)) {
-                        separator_iter = a;
-                } else if (path_is_separator (widget, model, path_b)) {
-                        separator_iter = b;
-                }
-
-                gtk_tree_path_free (path_a);
-                gtk_tree_path_free (path_b);
-        }
-
-        name_a = NULL;
-        is_separate_a = FALSE;
-        if (separator_iter != a) {
-                gtk_tree_model_get (model, a,
-                                    OPTION_NAME_COLUMN, &name_a,
-                                    OPTION_ITEM_IS_SEPARATED_COLUMN, &is_separate_a,
-                                    -1);
-        }
-
-        char *id;
-        name_b = NULL;
-        is_separate_b = FALSE;
-        if (separator_iter != b) {
-                gtk_tree_model_get (model, b,
-                                    OPTION_NAME_COLUMN, &name_b,
-                                    OPTION_ID_COLUMN, &id,
-                                    OPTION_ITEM_IS_SEPARATED_COLUMN, &is_separate_b,
-                                    -1);
-        }
-
-        if (widget->priv->separator_position == GDM_OPTION_WIDGET_POSITION_TOP) {
-                direction = -1;
+        if (is_top_separator) {
+                is_visible = widget->priv->number_of_top_rows > 0 &&
+                             widget->priv->number_of_middle_rows > 0;
+        } else if (is_bottom_separator) {
+                is_visible = widget->priv->number_of_bottom_rows > 0 &&
+                             widget->priv->number_of_middle_rows > 0;
         } else {
-                direction = 1;
+                is_visible = TRUE;
         }
 
-        if (separator_iter == b) {
-                result = is_separate_a? 1 : -1;
-                result *= direction;
-        } else if (separator_iter == a) {
-                result = is_separate_b? -1 : 1;
-                result *= direction;
-        } else if (is_separate_b == is_separate_a) {
-                result = g_utf8_collate (name_a, name_b);
+        gdm_option_widget_queue_visibility_check (widget);
+
+        return is_visible;
+}
+
+static int
+compare_item (GtkTreeModel *model,
+              GtkTreeIter  *a,
+              GtkTreeIter  *b,
+              gpointer      data)
+{
+        GdmOptionWidget *widget;
+        GtkTreePath     *path;
+        gboolean         a_is_separator;
+        gboolean         b_is_separator;
+        char            *name_a;
+        char            *name_b;
+        int              position_a;
+        int              position_b;
+        int              result;
+
+        g_assert (GDM_IS_OPTION_WIDGET (data));
+
+        widget = GDM_OPTION_WIDGET (data);
+
+        gtk_tree_model_get (model, a,
+                            OPTION_NAME_COLUMN, &name_a,
+                            OPTION_POSITION_COLUMN, &position_a,
+                            -1);
+
+        gtk_tree_model_get (model, b,
+                            OPTION_NAME_COLUMN, &name_b,
+                            OPTION_POSITION_COLUMN, &position_b,
+                            -1);
+
+        if (position_a != position_b) {
+                result = position_a - position_b;
+                goto out;
+        }
+
+        if (position_a == GDM_OPTION_WIDGET_POSITION_MIDDLE) {
+                a_is_separator = FALSE;
         } else {
-                result = is_separate_a - is_separate_b;
-                result *= direction;
+                path = gtk_tree_model_get_path (model, a);
+                a_is_separator = path_is_separator (widget, model, path);
+                gtk_tree_path_free (path);
         }
 
+        if (position_b == GDM_OPTION_WIDGET_POSITION_MIDDLE) {
+                b_is_separator = FALSE;
+        } else {
+                path = gtk_tree_model_get_path (model, b);
+                b_is_separator = path_is_separator (widget, model, path);
+                gtk_tree_path_free (path);
+        }
+
+        if (a_is_separator && b_is_separator) {
+                result = 0;
+                goto out;
+        }
+
+        if (!a_is_separator && !b_is_separator) {
+            result = g_utf8_collate (name_a, name_b);
+            goto out;
+        }
+
+        g_assert (position_a == position_b);
+        g_assert (position_a != GDM_OPTION_WIDGET_POSITION_MIDDLE);
+
+        result = a_is_separator - b_is_separator;
+
+        if (position_a == GDM_OPTION_WIDGET_POSITION_BOTTOM) {
+                result *= -1;
+        }
+out:
         g_free (name_a);
         g_free (name_b);
 
@@ -530,16 +763,31 @@ name_cell_data_func (GtkTreeViewColumn  *tree_column,
                      GdmOptionWidget   *widget)
 {
         char    *name;
+        char    *id;
         char    *markup;
+        gboolean is_default;
 
         name = NULL;
         gtk_tree_model_get (model,
                             iter,
+                            OPTION_ID_COLUMN, &id,
                             OPTION_NAME_COLUMN, &name,
                             -1);
 
-        markup = g_strdup_printf ("<span size='small'>%s</span>",
-                name ? name : "(null)");
+        if (widget->priv->default_item_id != NULL &&
+            id != NULL &&
+            strcmp (widget->priv->default_item_id, id) == 0) {
+                is_default = TRUE;
+        } else {
+                is_default = FALSE;
+        }
+        g_free (id);
+        id = NULL;
+
+        markup = g_strdup_printf ("<span size='small'>%s%s%s</span>",
+                                  is_default? "<i>" : "",
+                                  name ? name : "",
+                                  is_default? "</i>" : "");
         g_free (name);
 
         g_object_set (cell, "markup", markup, NULL);
@@ -552,17 +800,13 @@ separator_func (GtkTreeModel *model,
                 gpointer      data)
 {
         GdmOptionWidget *widget;
-        GtkTreePath      *path;
-        gboolean          is_separator;
+        GtkTreePath     *path;
+        gboolean         is_separator;
 
         g_assert (GDM_IS_OPTION_WIDGET (data));
 
         widget = GDM_OPTION_WIDGET (data);
 
-        if (widget->priv->separator_row == NULL) {
-                return FALSE;
-        }
-
         path = gtk_tree_model_get_path (model, iter);
 
         is_separator = path_is_separator (widget, model, path);
@@ -573,23 +817,34 @@ separator_func (GtkTreeModel *model,
 }
 
 static void
-add_separator (GdmOptionWidget *widget)
+add_separators (GdmOptionWidget *widget)
 {
         GtkTreeIter   iter;
         GtkTreeModel *model;
         GtkTreePath  *path;
 
-        g_assert (widget->priv->separator_row == NULL);
+        g_assert (widget->priv->top_separator_row == NULL);
+        g_assert (widget->priv->bottom_separator_row == NULL);
 
         model = GTK_TREE_MODEL (widget->priv->list_store);
 
         gtk_list_store_insert_with_values (widget->priv->list_store,
                                            &iter, 0,
-                                           OPTION_ID_COLUMN, "-",
-                                           OPTION_ITEM_IS_SEPARATED_COLUMN, TRUE,
+                                           OPTION_ID_COLUMN, "--",
+                                           OPTION_POSITION_COLUMN, GDM_OPTION_WIDGET_POSITION_BOTTOM,
                                            -1);
         path = gtk_tree_model_get_path (model, &iter);
-        widget->priv->separator_row =
+        widget->priv->bottom_separator_row =
+            gtk_tree_row_reference_new (model, path);
+        gtk_tree_path_free (path);
+
+        gtk_list_store_insert_with_values (widget->priv->list_store,
+                                           &iter, 0,
+                                           OPTION_ID_COLUMN, "-",
+                                           OPTION_POSITION_COLUMN, GDM_OPTION_WIDGET_POSITION_TOP,
+                                           -1);
+        path = gtk_tree_model_get_path (model, &iter);
+        widget->priv->top_separator_row =
             gtk_tree_row_reference_new (model, path);
         gtk_tree_path_free (path);
 }
@@ -610,32 +865,52 @@ gdm_option_widget_init (GdmOptionWidget *widget)
         gtk_container_add (GTK_CONTAINER (widget),
                            box);
 
+        widget->priv->image = gtk_image_new ();
+        gtk_widget_set_no_show_all (widget->priv->image, TRUE);
+        gtk_box_pack_start (GTK_BOX (box), widget->priv->image, FALSE, FALSE, 0);
+
         widget->priv->label = gtk_label_new ("");
         gtk_label_set_use_underline (GTK_LABEL (widget->priv->label), TRUE);
         gtk_label_set_use_markup (GTK_LABEL (widget->priv->label), TRUE);
-        gtk_widget_show (widget->priv->label);
-        gtk_container_add (GTK_CONTAINER (box), widget->priv->label);
+        gtk_widget_set_no_show_all (widget->priv->label, TRUE);
+        gtk_box_pack_start (GTK_BOX (box), widget->priv->label, FALSE, FALSE, 0);
 
         widget->priv->items_combo_box = gtk_combo_box_new ();
+
         g_signal_connect (widget->priv->items_combo_box,
                           "changed",
                           G_CALLBACK (on_changed),
                           widget);
 
-        gtk_widget_show (widget->priv->items_combo_box);
+        /* We disable the combo box until it has a default
+         */
+        gtk_widget_set_sensitive (widget->priv->items_combo_box, FALSE);
+        g_signal_connect (widget,
+                          "notify::default-item",
+                          G_CALLBACK (on_default_item_changed),
+                          NULL);
+
+        gtk_widget_set_no_show_all (widget->priv->items_combo_box, TRUE);
         gtk_container_add (GTK_CONTAINER (box),
                            widget->priv->items_combo_box);
         gtk_label_set_mnemonic_widget (GTK_LABEL (widget->priv->label),
-                                       widget->priv->items_combo_box);
+                                       GTK_WIDGET (widget));
 
         g_assert (NUMBER_OF_OPTION_COLUMNS == 4);
         widget->priv->list_store = gtk_list_store_new (NUMBER_OF_OPTION_COLUMNS,
                                                        G_TYPE_STRING,
                                                        G_TYPE_STRING,
-                                                       G_TYPE_BOOLEAN,
+                                                       G_TYPE_INT,
                                                        G_TYPE_STRING);
 
-        widget->priv->model_sorter = GTK_TREE_MODEL_SORT (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (widget->priv->list_store)));
+
+        widget->priv->model_filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (GTK_TREE_MODEL (widget->priv->list_store), NULL));
+
+        gtk_tree_model_filter_set_visible_func (widget->priv->model_filter,
+                                                check_item_visibilty,
+                                                widget, NULL);
+
+        widget->priv->model_sorter = GTK_TREE_MODEL_SORT (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (widget->priv->model_filter)));
 
         gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (widget->priv->model_sorter),
                                          OPTION_ID_COLUMN,
@@ -648,7 +923,7 @@ gdm_option_widget_init (GdmOptionWidget *widget)
         gtk_combo_box_set_model (GTK_COMBO_BOX (widget->priv->items_combo_box),
                                  GTK_TREE_MODEL (widget->priv->model_sorter));
 
-        add_separator (widget);
+        add_separators (widget);
         gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (widget->priv->items_combo_box),
                                               separator_func, widget, NULL);
 
@@ -674,6 +949,9 @@ gdm_option_widget_finalize (GObject *object)
 
         g_return_if_fail (widget->priv != NULL);
 
+        g_free (widget->priv->icon_name);
+        g_free (widget->priv->label_text);
+
         G_OBJECT_CLASS (gdm_option_widget_parent_class)->finalize (object);
 }
 
@@ -689,29 +967,38 @@ gdm_option_widget_new (const char *label_text)
 }
 
 void
-gdm_option_widget_add_item (GdmOptionWidget *widget,
-                             const char     *id,
-                             const char     *name,
-                             const char     *comment,
-                             gboolean        keep_separate)
+gdm_option_widget_add_item (GdmOptionWidget         *widget,
+                            const char              *id,
+                            const char              *name,
+                            const char              *comment,
+                            GdmOptionWidgetPosition  position)
 {
         GtkTreeIter iter;
 
         g_return_if_fail (GDM_IS_OPTION_WIDGET (widget));
 
-        if (keep_separate) {
-                widget->priv->number_of_separated_rows++;
-        } else {
-                widget->priv->number_of_normal_rows++;
+        switch (position) {
+            case GDM_OPTION_WIDGET_POSITION_BOTTOM:
+                widget->priv->number_of_bottom_rows++;
+                break;
+
+            case GDM_OPTION_WIDGET_POSITION_MIDDLE:
+                widget->priv->number_of_middle_rows++;
+                break;
+
+            case GDM_OPTION_WIDGET_POSITION_TOP:
+                widget->priv->number_of_top_rows++;
+                break;
         }
 
         gtk_list_store_insert_with_values (widget->priv->list_store,
                                            &iter, 0,
                                            OPTION_NAME_COLUMN, name,
                                            OPTION_COMMENT_COLUMN, comment,
-                                           OPTION_ITEM_IS_SEPARATED_COLUMN, keep_separate,
+                                           OPTION_POSITION_COLUMN, (int) position,
                                            OPTION_ID_COLUMN, id,
                                            -1);
+        gtk_tree_model_filter_refilter (widget->priv->model_filter);
 }
 
 void
@@ -720,7 +1007,7 @@ gdm_option_widget_remove_item (GdmOptionWidget *widget,
 {
         GtkTreeModel *model;
         GtkTreeIter   iter;
-        gboolean      is_separate;
+        int           position;
 
         g_return_if_fail (GDM_IS_OPTION_WIDGET (widget));
 
@@ -731,27 +1018,40 @@ gdm_option_widget_remove_item (GdmOptionWidget *widget,
                 return;
         }
 
-        is_separate = FALSE;
+        if (widget->priv->default_item_id != NULL &&
+            strcmp (widget->priv->default_item_id, id) == 0) {
+                g_critical ("Tried to remove default item from option widget");
+                return;
+        }
+
         gtk_tree_model_get (model, &iter,
-                            OPTION_ITEM_IS_SEPARATED_COLUMN, &is_separate,
+                            OPTION_POSITION_COLUMN, &position,
                             -1);
 
-        if (is_separate) {
-                widget->priv->number_of_separated_rows--;
-        } else {
-                widget->priv->number_of_normal_rows--;
+        switch ((GdmOptionWidgetPosition) position) {
+            case GDM_OPTION_WIDGET_POSITION_BOTTOM:
+                widget->priv->number_of_bottom_rows--;
+                break;
+
+            case GDM_OPTION_WIDGET_POSITION_MIDDLE:
+                widget->priv->number_of_middle_rows--;
+                break;
+
+            case GDM_OPTION_WIDGET_POSITION_TOP:
+                widget->priv->number_of_top_rows--;
+                break;
         }
 
         gtk_list_store_remove (widget->priv->list_store, &iter);
+        gtk_tree_model_filter_refilter (widget->priv->model_filter);
 }
 
-
 void
 gdm_option_widget_remove_all_items (GdmOptionWidget *widget)
 {
         GtkTreeIter   iter;
         GtkTreeModel *model;
-        gboolean      is_separated;
+        int           position;
         gboolean      is_valid;
 
         g_assert (GDM_IS_OPTION_WIDGET (widget));
@@ -764,10 +1064,10 @@ gdm_option_widget_remove_all_items (GdmOptionWidget *widget)
 
         do {
                 gtk_tree_model_get (model, &iter,
-                                    OPTION_ITEM_IS_SEPARATED_COLUMN, &is_separated,
+                                    OPTION_POSITION_COLUMN, &position,
                                     -1);
 
-                if (!is_separated) {
+                if ((GdmOptionWidgetPosition) position == GDM_OPTION_WIDGET_POSITION_MIDDLE) {
                         is_valid = gtk_list_store_remove (widget->priv->list_store,
                                                           &iter);
                 } else {
@@ -779,20 +1079,16 @@ gdm_option_widget_remove_all_items (GdmOptionWidget *widget)
 }
 
 gboolean
-gdm_option_widget_lookup_item (GdmOptionWidget *widget,
-                                const char       *id,
-                                char            **name,
-                                char            **comment,
-                                gboolean         *is_separate)
+gdm_option_widget_lookup_item (GdmOptionWidget          *widget,
+                               const char               *id,
+                               char                    **name,
+                               char                    **comment,
+                               GdmOptionWidgetPosition  *position)
 {
         GtkTreeIter   iter;
         char         *active_item_id;
 
         g_return_val_if_fail (GDM_IS_OPTION_WIDGET (widget), FALSE);
-
-        if (id == NULL) {
-                sleep (30);
-        }
         g_return_val_if_fail (id != NULL, FALSE);
 
         active_item_id = get_active_item_id (widget, &iter);
@@ -817,21 +1113,14 @@ gdm_option_widget_lookup_item (GdmOptionWidget *widget,
                                     OPTION_COMMENT_COLUMN, comment, -1);
         }
 
-        if (is_separate != NULL) {
+        if (position != NULL) {
+                int position_as_int;
+
                 gtk_tree_model_get (GTK_TREE_MODEL (widget->priv->list_store), &iter,
-                                    OPTION_ITEM_IS_SEPARATED_COLUMN, is_separate, -1);
+                                    OPTION_POSITION_COLUMN, &position_as_int, -1);
+
+                *position = (GdmOptionWidgetPosition) position_as_int;
         }
 
         return TRUE;
 }
-
-void
-gdm_option_widget_set_separator_position (GdmOptionWidget         *widget,
-                                          GdmOptionWidgetPosition  position)
-{
-        g_return_if_fail (GDM_IS_OPTION_WIDGET (widget));
-
-        if (widget->priv->separator_position != position) {
-                widget->priv->separator_position = position;
-        }
-}
