@@ -35,7 +35,6 @@
 #include <bonobo/bonobo-ui-util.h>
 
 #include <libgnome/gnome-init.h>
-#include <libgnomeui/gnome-help.h>
 
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
@@ -69,6 +68,7 @@ typedef struct _GdmAppletData
         guint           user_notify_id;
         GQuark          user_menu_item_quark;
         gint8           pixel_size;
+        gint            panel_size;
         GtkIconSize     icon_size;
 } GdmAppletData;
 
@@ -80,6 +80,8 @@ typedef struct _SelectorResponseData
 
 static GtkTooltips *tooltips = NULL;
 
+static void reset_icon (GdmAppletData *adata);
+
 static gboolean applet_factory (PanelApplet   *applet,
                                 const char    *iid,
                                 gpointer       data);
@@ -130,23 +132,6 @@ menubar_button_press_event_cb (GtkWidget      *menubar,
         return FALSE;
 }
 
-static void
-help_cb (BonoboUIComponent *ui_container,
-         GdmAppletData     *adata,
-         const char        *cname)
-{
-        GError *err;
-
-        err = NULL;
-        gnome_help_display_on_screen ("gdm-user-switch-applet", NULL,
-                                      gtk_widget_get_screen (GTK_WIDGET (adata->applet)),
-                                      &err);
-        if (err != NULL) {
-                g_warning ("Could not open help document: %s", err->message);
-                g_error_free (err);
-        }
-}
-
 static void
 about_cb (BonoboUIComponent *ui_container,
           gpointer           data,
@@ -415,7 +400,6 @@ applet_size_allocate_cb (GtkWidget     *widget,
         GList            *children;
         GtkWidget        *top_item;
         PanelAppletOrient orient;
-        gint              item_border;
         gint              pixel_size;
         gdouble           text_angle;
         GtkPackDirection  pack_direction;
@@ -433,18 +417,15 @@ applet_size_allocate_cb (GtkWidget     *widget,
 
         orient = panel_applet_get_orient (PANEL_APPLET (widget));
 
-        item_border = (MAX (top_item->style->xthickness,
-                            top_item->style->ythickness) * 2);
-
         switch (orient) {
         case PANEL_APPLET_ORIENT_UP:
         case PANEL_APPLET_ORIENT_DOWN:
                 gtk_widget_set_size_request (top_item, -1, allocation->height);
-                pixel_size = allocation->height - item_border;
+                pixel_size = allocation->height - top_item->style->ythickness * 2;
                 break;
         case PANEL_APPLET_ORIENT_LEFT:
                 gtk_widget_set_size_request (top_item, allocation->width, -1);
-                pixel_size = allocation->width - item_border;
+                pixel_size = allocation->width - top_item->style->xthickness * 2;
                 pack_direction = GTK_PACK_DIRECTION_TTB;
                 text_angle = 270.0;
                 text_xalign = 0.5;
@@ -452,7 +433,7 @@ applet_size_allocate_cb (GtkWidget     *widget,
                 break;
         case PANEL_APPLET_ORIENT_RIGHT:
                 gtk_widget_set_size_request (top_item, allocation->width, -1);
-                pixel_size = allocation->width - item_border;
+                pixel_size = allocation->width - top_item->style->xthickness * 2;
                 pack_direction = GTK_PACK_DIRECTION_BTT;
                 text_angle = 90.0;
                 text_xalign = 0.5;
@@ -472,6 +453,11 @@ applet_size_allocate_cb (GtkWidget     *widget,
                                            text_angle,
                                            text_xalign,
                                            text_yalign);
+
+        if (adata->panel_size != pixel_size) {
+                adata->panel_size = pixel_size;
+                reset_icon (adata);
+        }
 }
 
 
@@ -666,7 +652,7 @@ menuitem_destroy_cb (GtkWidget *menuitem,
                      gpointer   data)
 {
         GdmAppletData *adata;
-        GSList *li;
+        GSList        *li;
 
         adata = data;
 
@@ -680,6 +666,7 @@ menuitem_destroy_cb (GtkWidget *menuitem,
                 }
         }
 
+        g_debug ("Menuitem destroyed - removing");
         li = g_slist_find (adata->items, menuitem);
         adata->items = g_slist_delete_link (adata->items, li);
         sort_menu (adata);
@@ -960,6 +947,96 @@ login_screen_activate_cb (GtkMenuItem *item,
         do_switch (adata, user);
 }
 
+static void
+create_sub_menu (GdmAppletData *adata)
+{
+        GSList *users;
+
+        adata->menu = gtk_menu_new ();
+        gtk_menu_item_set_submenu (GTK_MENU_ITEM (adata->menuitem), adata->menu);
+        g_signal_connect (adata->menu, "style-set",
+                          G_CALLBACK (menu_style_set_cb), adata);
+        g_signal_connect (adata->menu, "show",
+                          G_CALLBACK (menu_expose_cb), adata);
+        gtk_widget_show (adata->menu);
+
+
+        /* This next part populates the list with all the users we currently know
+         * about. For almost all cases, this is the empty list, because we're
+         * asynchronous, and the data hasn't come back from the callback saying who
+         * the users are yet. However, if someone has two GDMs on their toolbars (why,
+         * I have no freaking idea, but bear with me here), the menu of the second
+         * one to be initialised needs to be filled in from the start rather than
+         * depending on getting data from the callback like the first one.
+         */
+        users = gdm_user_manager_list_users (adata->manager);
+        while (users != NULL) {
+                add_user (adata, users->data);
+                update_user_item_visibility (adata, users->data);
+
+                users = g_slist_delete_link (users, users);
+        }
+
+        g_signal_connect (adata->manager,
+                          "user-added",
+                          G_CALLBACK (on_manager_user_added),
+                          adata);
+        g_signal_connect (adata->manager,
+                          "user-is-logged-in-changed",
+                          G_CALLBACK (on_manager_user_is_logged_in_changed),
+                          adata);
+
+        adata->separator_item = gtk_separator_menu_item_new ();
+        gtk_menu_shell_append (GTK_MENU_SHELL (adata->menu), adata->separator_item);
+        g_signal_connect (adata->separator_item, "destroy",
+                          G_CALLBACK (menuitem_destroy_cb), adata);
+        adata->items = g_slist_prepend (adata->items, adata->separator_item);
+        gtk_widget_show (adata->separator_item);
+
+        adata->login_screen_item = gtk_image_menu_item_new_with_label (_("Other..."));
+        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (adata->login_screen_item),
+                                       gtk_image_new ());
+        gtk_menu_shell_append (GTK_MENU_SHELL (adata->menu),
+                               adata->login_screen_item);
+        g_signal_connect (adata->login_screen_item, "style-set",
+                          G_CALLBACK (menuitem_style_set_cb), adata);
+        g_signal_connect (adata->login_screen_item, "destroy",
+                          G_CALLBACK (menuitem_destroy_cb), adata);
+        g_signal_connect (adata->login_screen_item, "activate",
+                          G_CALLBACK (login_screen_activate_cb), adata);
+        adata->items = g_slist_prepend (adata->items, adata->login_screen_item);
+        gtk_widget_show (adata->login_screen_item);
+
+        adata->items = g_slist_sort_with_data (adata->items,
+                                               sort_menu_comparedatafunc,
+                                               adata);
+}
+
+static void
+destroy_sub_menu (GdmAppletData *adata)
+{
+        gtk_menu_item_set_submenu (GTK_MENU_ITEM (adata->menuitem), NULL);
+
+        g_signal_handlers_disconnect_by_func (adata->manager,
+                                              G_CALLBACK (on_manager_user_added),
+                                              adata);
+        g_signal_handlers_disconnect_by_func (adata->manager,
+                                              G_CALLBACK (on_manager_user_is_logged_in_changed),
+                                              adata);
+}
+
+static void
+set_menu_visibility (GdmAppletData *adata,
+                     gboolean       visible)
+{
+
+        if (visible) {
+                create_sub_menu (adata);
+        } else {
+                destroy_sub_menu (adata);
+        }
+}
+
 static void
 client_notify_lockdown_func (GConfClient   *client,
                              guint          cnxn_id,
@@ -972,14 +1049,15 @@ client_notify_lockdown_func (GConfClient   *client,
         value = gconf_entry_get_value (entry);
         key = gconf_entry_get_key (entry);
 
-        if (!value || !key)
+        if (value == NULL || key == NULL) {
                 return;
+        }
 
         if (strcmp (key, LOCKDOWN_KEY) == 0) {
                 if (gconf_value_get_bool (value)) {
-                        gtk_widget_hide ( GTK_WIDGET (adata->applet));
+                        set_menu_visibility (adata, FALSE);
                 } else {
-                        gtk_widget_show ( GTK_WIDGET (adata->applet));
+                        set_menu_visibility (adata, TRUE);
                 }
         }
 }
@@ -994,7 +1072,7 @@ reset_icon (GdmAppletData *adata)
                 return;
         }
 
-        pixbuf = gdm_user_render_icon (adata->user, 24);
+        pixbuf = gdm_user_render_icon (adata->user, adata->panel_size);
         if (pixbuf != NULL) {
                 image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (adata->menuitem));
                 gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
@@ -1051,12 +1129,10 @@ fill_applet (PanelApplet *applet)
         static const BonoboUIVerb menu_verbs[] = {
                 BONOBO_UI_VERB ("GdmAboutMe", about_me_cb),
                 BONOBO_UI_VERB ("GdmUsersGroupsAdmin", admin_cb),
-                BONOBO_UI_VERB ("GdmHelp", (BonoboUIVerbFn)help_cb),
                 BONOBO_UI_VERB ("GdmAbout", about_cb),
                 BONOBO_UI_VERB_END
         };
         static gboolean    first_time = FALSE;
-        GSList            *users;
         char              *tmp;
         BonoboUIComponent *popup_component;
         GdmAppletData     *adata;
@@ -1083,12 +1159,12 @@ fill_applet (PanelApplet *applet)
 
         adata = g_new0 (GdmAppletData, 1);
         adata->applet = applet;
+        adata->panel_size = 24;
 
         /* Until we add user selecting to GDM */
         adata->active_only = TRUE;
 
         adata->client = gconf_client_get_default ();
-        adata->manager = gdm_user_manager_ref_default ();
 
         tmp = g_strdup_printf ("applet-user-menu-item-%p", adata);
         adata->user_menu_item_quark = g_quark_from_string (tmp);
@@ -1183,62 +1259,9 @@ fill_applet (PanelApplet *applet)
         gtk_container_add (GTK_CONTAINER (applet), adata->menubar);
         gtk_widget_show (adata->menubar);
 
+        adata->manager = gdm_user_manager_ref_default ();
         setup_current_user (adata);
 
-
-
-        adata->menu = gtk_menu_new ();
-        gtk_menu_item_set_submenu (GTK_MENU_ITEM (adata->menuitem), adata->menu);
-        g_signal_connect (adata->menu, "style-set",
-                          G_CALLBACK (menu_style_set_cb), adata);
-        g_signal_connect (adata->menu, "show",
-                          G_CALLBACK (menu_expose_cb), adata);
-        gtk_widget_show (adata->menu);
-
-        /* This next part populates the list with all the users we currently know
-         * about. For almost all cases, this is the empty list, because we're
-         * asynchronous, and the data hasn't come back from the callback saying who
-         * the users are yet. However, if someone has two GDMs on their toolbars (why,
-         * I have no freaking idea, but bear with me here), the menu of the second
-         * one to be initialised needs to be filled in from the start rather than
-         * depending on getting data from the callback like the first one.
-         */
-        users = gdm_user_manager_list_users (adata->manager);
-        while (users != NULL) {
-                add_user (adata, users->data);
-                update_user_item_visibility (adata, users->data);
-
-                users = g_slist_delete_link (users, users);
-        }
-
-        g_signal_connect (adata->manager,
-                          "user-added",
-                          G_CALLBACK (on_manager_user_added),
-                          adata);
-        g_signal_connect (adata->manager,
-                          "user-is-logged-in-changed",
-                          G_CALLBACK (on_manager_user_is_logged_in_changed),
-                          adata);
-
-        adata->separator_item = gtk_separator_menu_item_new ();
-        gtk_menu_shell_append (GTK_MENU_SHELL (adata->menu), adata->separator_item);
-        adata->items = g_slist_prepend (adata->items, adata->separator_item);
-        gtk_widget_show (adata->separator_item);
-
-        adata->login_screen_item = gtk_image_menu_item_new_with_label (_("Other..."));
-        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (adata->login_screen_item),
-                                       gtk_image_new ());
-        gtk_menu_shell_append (GTK_MENU_SHELL (adata->menu),
-                               adata->login_screen_item);
-        g_signal_connect (adata->login_screen_item, "style-set",
-                          G_CALLBACK (menuitem_style_set_cb), adata);
-        g_signal_connect (adata->login_screen_item, "destroy",
-                          G_CALLBACK (menuitem_destroy_cb), adata);
-        g_signal_connect (adata->login_screen_item, "activate",
-                          G_CALLBACK (login_screen_activate_cb), adata);
-        adata->items = g_slist_prepend (adata->items, adata->login_screen_item);
-        gtk_widget_show (adata->login_screen_item);
-
         adata->client_notify_lockdown_id = gconf_client_notify_add (adata->client,
                                                                     LOCKDOWN_KEY,
                                                                     (GConfClientNotifyFunc)client_notify_lockdown_func,
@@ -1246,16 +1269,14 @@ fill_applet (PanelApplet *applet)
                                                                     NULL,
                                                                     NULL);
 
-        adata->items = g_slist_sort_with_data (adata->items,
-                                               sort_menu_comparedatafunc,
-                                               adata);
-
         if (gconf_client_get_bool (adata->client, LOCKDOWN_KEY, NULL)) {
-                gtk_widget_hide (GTK_WIDGET (applet));
+                set_menu_visibility (adata, FALSE);
         } else {
-                gtk_widget_show (GTK_WIDGET (applet));
+                set_menu_visibility (adata, TRUE);
         }
 
+        gtk_widget_show (GTK_WIDGET (adata->applet));
+
         return TRUE;
 }
 
